#!/bin/bash

# Description: 指定した期間の休日を抽出する
# Create: 2021-07-07
# Update: 2022-03-14

our_holidays="
20221230
20221231
20230101
20230102
20230103
"

usage_exit() {
    echo " Usage: $0 [-msnc] [開始日 [終了日]]"
    echo "   -m : our_holidaysも含める"
    echo "   -s : 祝日の説明あり"
    echo "   -n : カレンダーデータを保存しない"
    echo "   -c : カレンダーデータがあるならネットワークから取得しない"
    echo "   日にち指定 [20220201 [20220331]]"
    exit
}

_pcre() { # PCREを用いる
    case "$(uname -a)" in
        *Darwin*)
            if [ -x "/usr/local/bin/pcre2grep" ]; then
                PCREGREP="pcre2grep"
            elif [ -x "/usr/local/bin/pcregrep" ]; then
                PCREGREP="pcregrep"
            elif [ -n "$(which ggrep)" ]; then
                PCREGREP="ggrep -P"
            fi
            ;;
        *)        PCREGREP="grep -P" ;;
    esac
    if ! $PCREGREP '' <<<"" > /dev/null 2>&1  ;then
        # unset PCREGREP
        echo "no PCRE function"
        exit
    fi
}

readcalendar() {
    if [ $save_calendar -ne 0 ]; then
        wget -N -q https://calendar.google.com/calendar/ical/ja.japanese%23holiday%40group.v.calendar.google.com/public/basic.ics
        cat basic.ics
    elif [ $get_calendar -eq 0 -a -f "./basic.ics" ]; then
        cat basic.ics
    else
        wget -q https://calendar.google.com/calendar/ical/ja.japanese%23holiday%40group.v.calendar.google.com/public/basic.ics  -O -
    fi
}

holidays() {
    readcalendar |
    nkf -Lu |
    $PCREGREP -o $query | # PCREがない環境では、egrep+sedで置き換える
    $PASTE |
    sort -n -k 1,1 |
    awk -v b=${1:-00000000} -v e=${2:-99999999} '$1>=b&&$1<=e'
}

with_summary=0
query='(?<=^DTSTART;VALUE=DATE:).*' 
PASTE="cat"
merge_our_holiday=0
fmt=$'%s\n'
save_calendar=0
get_calendar=0

while getopts :mcns OPT ; do
  case $OPT in
    c) save_calendar=1 ;;
    n) get_calendar=0 ;;
    s)  with_summary=1
        query='(?<=^DTSTART;VALUE=DATE:|^SUMMARY:).*' 
        PASTE="paste - -"
        ;;
    m) merge_our_holiday=1 ;;
    *) usage_exit ;;
  esac
done
shift $((OPTIND - 1))

_pcre
if [ $merge_our_holiday -eq 0 ]; then
    holidays $@
else
    [ $with_summary -eq 1 ] && fmt=$'%s\tOUR HOLIDAY\n'
    join -t $'\t' -a 1 -a 2 <(holidays $@) <(printf "$fmt" $our_holidays)
fi
exit

if [ $with_summary -gt 0 ]; then
    if [ $merge_our_holiday -eq 0 ]; then
        with $@
    else
        join -t $'\t' -a 1 -a 2 <(with $@) <(printf "%s\tOUR HOLIDAY\n" $our_holidays)
    fi
else
    if [ $merge_our_holiday -eq 0 ]; then
        without $@
    else
        join -t $'\t' -a 1 -a 2 <(without $@) <(printf "%s\n" $our_holidays)
    fi
fi


